
var app = angular.module('TabletsGame', [
  'ui.router',
  'mobile-angular-ui',
  'mobile-angular-ui.gestures',
  'ngSanitize'
]);

app.run(function($rootScope, $location) {
  $rootScope.config = {
    // язык (en / ru)
    "lang": "en",
    // операционная система (ios / android)
    "os": "android",
    // время ожидания результатов в мс (для интриги)
    "resultsWaitingTime": 800,
    // звук (вкл / выкл)
    "sound": true,
    // номер раунда должен быть кратен этому числу, чтобы показалась реклама
    "adDivider": 4
  };


  function loadSettingsFromLocalStorage() {
    if (localStorage.getItem('tabletsGameLang') != undefined) {
      if (localStorage.getItem('tabletsGameLang') == 'ru' || localStorage.getItem('tabletsGameLang') == 'en') {
        $rootScope.config.lang = localStorage.getItem('tabletsGameLang');
      }
    }
    else {
      // получить язык по региону
      var regionLanguage = 'ru';
      localStorage.setItem('tabletsGameLang', regionLanguage);
    }

    if (localStorage.getItem('tabletsGameSound') == 'false') {
      $rootScope.config.sound = false;
    }   
  }
  loadSettingsFromLocalStorage();


  var getPhoneGapLocaleName = function ( callback ) {
    var unknownLocation = 'unknown'; //Default value
    if (navigator.globalization !== undefined) {
      navigator.globalization.getLocaleName(
        function (locale) {
          if (locale !== undefined) {
            callback(locale.value);
          }
          else {
            callback(unknownLocation);
          }
        },
        function () {
          callback(unknownLocation);
        }
      );
    }
    else {
      callback(unknownLocation);
    }
  };

  // если первый раз запущено, приложение подбирает подходящий язык
  if (localStorage.getItem('tabletsGameLang') == undefined) {
    getPhoneGapLocaleName(function(loc){ 
      var russian = [
        "ru", "mo", "be", "by", "kz", "kg", "ge", "ua", "md", "tj", "uz", "lt", "ltu", "lv", "lat"
      ];
      var isRussian = false;
      for (var i = 0; i < russian.length; i++) {
        if (loc.toLowerCase().indexOf(russian[i]) >= 0) {
          isRussian = true;
          break;
        }
      }
      if (isRussian) {
        $rootScope.config.lang = 'ru';
        localStorage.setItem('tabletsGameLang', 'ru');
      }
      else {
        $rootScope.config.lang = 'en';
        localStorage.setItem('tabletsGameLang', 'en');
      }
    });
  }


  $rootScope.getAdType = function() {
    // реклама в процентах (в сумме 100), не обязательно 2 типа, можно и больше
    var traffic = [
      // картиночный межстраничник
      {
        "type": "intersistial",
        "percent": 90,
      },
      // своя реклама с просьбой поделиться и т.д.
      {
        "type": "self",
        "percent": 10
      }
    ];
    var randomNumber = Math.floor(Math.random() * 100) + 1; // рандомное число от 1 до 100
    var leftBound = 0;
    var rightBound = traffic[0].percent;
    var i = 0; // индекс подходящего трафика
    while (randomNumber <= leftBound || randomNumber > rightBound) {
      i++;
      leftBound = 0;
      rightBound = traffic[0].percent;
      // левая граница
      for (var j = 0; j < i - 1; j++) {
        leftBound += traffic[j].percent;
      }
      // правая граница
      for (var j = 1; j < i; j++) {
        rightBound += traffic[j].percent;
      }
    }
    var sumForRightBound = 0;
    for (var i = 0; i < traffic.length; i++) {
      sumForRightBound += traffic[i].percent;
      if (sumForRightBound == rightBound) {
        return traffic[i].type;
      }
    }
  };


  if (window.cordova) {
    if ($rootScope.config.os == 'ios') {
      // admob
      //admob.initAdmob('ca-app-pub-3940256099942544/6300978111', 'ca-app-pub-3940256099942544/1033173712');
      admob.initAdmob('ca-app-pub-6369492337010523/7520363892', 'ca-app-pub-6369492337010523/3804638291');
      // аналитика
      //window.analytics.startTrackerWithId('UA-80922967-1');
    }
    else if ($rootScope.config.os == 'android') {
      // admob
      //admob.initAdmob('ca-app-pub-3940256099942544/6300978111', 'ca-app-pub-3940256099942544/1033173712');
      admob.initAdmob('ca-app-pub-6369492337010523/4566897498', 'ca-app-pub-6369492337010523/6043630690');
      // аналитика
      //window.analytics.startTrackerWithId('UA-80922967-2');      
    }
  }

  $rootScope.string = {
    "menu_start": {
      "ru": "Старт",
      "en": "Play"
    },
    "menu_rules": {
      "ru": "Правила",
      "en": "Rules"
    },


    "sound_on": {
      "ru": "Включить звук",
      "en": "Sound on"
    },
    "sound_off": {
      "ru": "Выключить звук",
      "en": "Sound off"
    },
    "remove_ad": {
      "ru": "Отключить рекламу",
      "en": "Block ad"
    },
    "rate": {
      "ru": "Оценить игру",
      "en": "Rate the game"
    },
    "social_share": {
      "ru": "Рассказать друзьям",
      "en": "Share it"
    },
    "credits_btn": {
      "ru": "Благодарности",
      "en": "Credits"
    },


    "how_you_rate": {
      "ru": "Как бы вы оценили игру?",
      "en": "How would you rate us?"
    },
    "rate_hate_desc": {
      "ru": "Напишите, пожалуйста, что вам не понравилось в игре. Надеемся, ваш отзыв поможет нам сделать из игры что-то более хорошее",
      "en": "Tell us what you didn't like. Everyone's opinion helps us to improve the game."
    },
    "send": {
      "ru": "Отправить",
      "en": "Send"
    },
    "rate_results_text": {
      "ru": "Большое спасибо за отзыв, он нам очень поможет!",
      "en": "Thank you for your feedback, it will help us."
    },



    "rules_skip": {
      "ru": "Пропустить",
      "en": "Skip"
    },
    "rules_start": {
      "ru": "Начать игру",
      "en": "Start"
    },
    "rules_slide1_text": {
      "ru": "У вас 2 таблетки: хорошая и плохая. Съедите хорошую - будете жить, съедите плохую - умрете",
      "en": "There's a good pill and a bad pill. You take the good pill - you live, take the bad pill - you die."
    },
    "rules_slide2_text": {
      "ru": "Вы знаете, где какая таблетка, и предлагаете другому человеку сыграть в игру: одновременно съесть по одной. Вы кладете таблетку себе и таблетку сопернику (сами решаете, кому какую)",
      "en": "You know where the bad and where the good pill is and you offer your rival to play the game: to eat the medecine together at the same time. You take the one pill and give the other to your rival (its up to you which one you will choose)"
    },
    "rules_slide3_text": {
      "ru": "Ваш соперник может выбрать любую из двух таблеток. А вы должны съесть оставшуюся. Один из вас умирает.<br>После этого вы с соперником меняетесь ролями: теперь вам придется выбирать, какую есть.",
      "en": "Your rival can choose either of pills. And you have to eat the other one. Only one survivor!!! Then you change your roles: now you have to choose which one you will eat"
    },
    "rules_slide4_text": {
      "ru": 'Не играйте в случайность! Соперник может задавать любые уточняющие вопросы, к примеру, может спросить: "Здесь ли хорошая таблетка?" Запутайте его любой ценой!',
      "en": "Dont leave anything to chance!!! Your rival can ask for any clarification. For example, he can ask if the good pill is here (or maybe it is there). Puzzle him by all means!!!"
    },
    "ideas_to_punish": {
      "ru": "На что играть?",
      "en": "5 ideas to punish the loser"
    },

    "tips_slide1_title": {
      "ru": "Умерший пьет",
      "en": "Dead and drunk"
    },
    "tips_slide1_text": {
      "ru": "Игрок выпивает рюмку, умирая каждые 3 раза",
      "en": "The player who dies 3 times drinks a short of vodka (or any alcohol)"
    },
    "tips_slide2_title": {
      "ru": "Игра на желание",
      "en": "Making a wish"
    },
    "tips_slide2_text": {
      "ru": "Тот, кто первым умрет 5 раз, должен будет исполнить желаение второго игрока.<br>После исполнения желения игра продолжается в том же духе",
      "en": "The player who dies first five times must fulfill the wish of his rival. When the wish is fulfilled, the game is going on, keep it up!!"
    },
    "tips_slide3_title": {
      "ru": "Игра на щелбаны",
      "en": "Fillips"
    },
    "tips_slide3_text": {
      "ru": "Каждый раз игрок, которому достается плохая таблетка, получает хороший щелбан от противника",
      "en": "Every time the player who gets bad pill is filliped by his lucky rival!!!"
    },
    "tips_slide4_title": {
      "ru": "Проигравший отжимается",
      "en": "Press up"
    },
    "tips_slide4_text": {
      "ru": "Игроку, умершему 5 раз, приходится отжиматься / подтягиваться установленное количество раз",
      "en": "A player who dies five times has to do press-up and pull-up fixed number of times."
    },
    "tips_slide5_title": {
      "ru": "Придумайте сами",
      "en": "Invent by yourself"
    },
    "tips_slide5_text": {
      "ru": "Удары небольшим разрядом электричества из пьезоэлемента зажигалки, поедание чего-то противного, - что только придумаете!",
      "en": "You can shock slightly your rival with piezoelectric element of cigarette-lighter or you can make him eat smth disgusting - let your imagination go!"
    },

    "self_ad_text": {
      "ru": "Не могли бы вы рассказать о приложении своим друзьям? Вы нам очень поможете",
      "en": "Would you be so kind to share our game with your friends? It will help us."
    },
    

    "entering_player1_name": {
      "ru": "Имя первого игрока:",
      "en": "Player 1:"
    },
    "entering_player2_name": {
      "ru": "Имя второго игрока:",
      "en": "Player 2:"
    },
    "entering_player1_placeholder": {
      "ru": "Например: Саша",
      "en": "For example: Alex"
    },
    "entering_player2_placeholder": {
      "ru": "Например: Маша",
      "en": "For example: Mary"
    },
    "entering_continue": {
      "ru": "Продолжить",
      "en": "Continue"
    },
    "player_warning_text_part1": {
      "ru": ", ты будешь знать, где какая таблетка, и тебе предстоит расположить их по своему усмотрению.<br>Убедись, что ",
      "en": ", you will know where the bad and where the good pill is, and it depends on you how to dispose them. Make sure that the "
    },
    "player_warning_text_part2": {
      "ru": " не подсматривает!",
      "en": " dont pry!",
    },



    "game_your_action": {
      "ru": ", твой ход",
      "en": ", its your turn"
    },
    "game_tablet_choosing_text": {
      "ru": "Какую съешь?",
      "en": "Which one?"
    },
    "game_player_dying": {
      "ru": " умирает",
      "en": " dies"
    },
    "game_continue": {
      "ru": "Продолжить",
      "en": "Continue"
    },
    "game_players_changing_text_part1": {
      "ru": ", передай телефон. Теперь ",
      "en": ", hand the mobile phone to the "
    },
    "game_players_changing_text_part2": {
      "ru": " будет знать, где какая таблетка!",
      "en": ". Its high time for "
    },
    "game_players_changing_text_part3": {
      "ru": "",
      "en": " to dispose the pills."
    },

    "share_options_message": {
      "ru": "Почувствуй себя Шерлоком, сыграв шахматную партию в один ход до смерти!",
      "en": "Feel yourself being Sherlock Holmes, playing a game of chess, with one move to death!"
    },
    "share_options_subject": {
      "ru": "Death Pill - скачать бесплатно",
      "en": "Death Pill - Free Game"
    },
    "share_options_chooser_title": {
      "ru": "Death Pill - 1 партия, 1 ход, 1 выживший",
      "en": "Death Pill - 1 game, 1 move, 1 survivor"
    }

  };

  $rootScope.shareWithFriends = function() {
    var options = {
      message: $rootScope.string.share_options_message[$rootScope.config.lang],
      subject: $rootScope.string.share_options_subject[$rootScope.config.lang], // for email
      files: ['http://deathpill.lol/img/logo.png'], 
      url: 'http://deathpill.lol',
      chooserTitle: $rootScope.string.share_options_chooser_title[$rootScope.config.lang]
    };
    if (window.cordova) {
      window.plugins.socialsharing.shareWithOptions(options, null, null);
      //window.analytics.trackEvent('Events', 'Share with friends');
    }
  };



  if (window.cordova) {
    var defaultPath = '';
    if ($rootScope.config.os == 'android') {
      defaultPath = '/android_asset/www/'; // file:/*/*/*android_asset/www/ 
    }

    var loopBgSound = function(status) {
      if (status === Media.MEDIA_STOPPED) {
        $rootScope.sound.bg.play();
      }
    };
    $rootScope.sound = {};
    $rootScope.sound.bg = new Media(defaultPath + 'dist/audio/bg.mp3', null, null, loopBgSound);
    $rootScope.sound.dead = new Media(defaultPath + 'dist/audio/dead.mp3');
    $rootScope.sound.alive = new Media(defaultPath + 'dist/audio/alive.mp3');
    $rootScope.sound.round = new Media(defaultPath + 'dist/audio/round.mp3');

    $rootScope.sound.bg.play();
  }


  if (localStorage.getItem('TabletsGameIsFirstTime') != 'false') {
    localStorage.setItem('TabletsGameIsFirstTime', 'false');
    $location.path('/rules');
  }

});


app.config(function($stateProvider, $urlRouterProvider){
  $urlRouterProvider.otherwise("/");
  $stateProvider
  .state('menu', {
    url: '/',
    templateUrl: 'partials/menu.html',
    controller: 'MenuController'
  })
  .state('game', {
    url: '/game',
    templateUrl: 'partials/game.html',
    controller: 'GameController'
  })
  .state('rules', {
    url: '/rules',
    templateUrl: 'partials/rules.html',
    controller: 'RulesController'
  })
  .state('tips', {
    url: '/tips',
    templateUrl: 'partials/tips.html',
    controller: 'TipsController'
  })
  .state('settings', {
    url: '/settings',
    templateUrl: 'partials/settings.html',
    controller: 'SettingsController'
  }) 
  .state('rate', {
    url: '/rate',
    templateUrl: 'partials/rate.html',
    controller: 'RateController'
  })
  .state('credits', {
    url: '/credits',
    templateUrl: 'partials/credits.html'
  })
});


// Кнопка "Назад"
app.directive('back', ['$window', function($window) {
  return {
    restrict: 'A',
    link: function (scope, elem, attrs) {
      elem.bind('click', function () {
        history.go(-1);
      });
    }
  };
}]);


// убирает badparse в bind-html
app.filter('toTrusted', function ($sce) {
  return function (value) {
      return $sce.trustAsHtml(value);
  };
});


app.controller('MenuController', function($scope, $rootScope){
  if (window.cordova) {
    if ($rootScope.config.sound) {
      $rootScope.sound.bg.setVolume(1.0);
    }
    else {
      $rootScope.sound.bg.setVolume(0.0);
    }
  }

  $scope.exit = function() {
    if (window.cordova) {
      if (navigator.app) {
        navigator.app.exitApp();
      }
      else if (navigator.device) {
        navigator.device.exitApp();
      }
    }
  };

  if (window.cordova) {
    //window.analytics.trackView('Menu');
  }
});


app.controller('GameController', function($scope, $rootScope, $timeout, $state) {

  $scope.players = [
    // 1 игрок
    {
      "score": 0,
      "dead": false,
      "name": "",
      "defaultName": {
        "ru": "Игрок 1",
        "en": "Player 1"
      }
    },

    // 2 игрок
    {
      "score": 0,
      "dead": false,
      "name": "",
      "defaultName": {
        "ru": "Игрок 2",
        "en": "Player 2"
      }
    }
  ];

  $scope.currentPlayerIndex = 0; // индекс игрока, который распределяет таблетки
  $scope.currentRound = 1; // текущий раунд
  var currentAdType = ''; // тип текущей рекламы

  $scope.gameStates = {
    "list": {
      "namesEntering": true,
      "firstPlayerScreen": false,
      "selfAd": false,
      "tabletsPositions": false,
      "tabletChoosing": false,
      "waitingResults": false,
      "gameResults": false,
      "playersChanging": false
    },
    set: function(stateName) {
      for (var state in $scope.gameStates.list) {
        $scope.gameStates.list[state] = false;
      }
      $scope.gameStates.list[stateName] = true;
    }
  };

  $scope.tablets = {
    "intoxicated": {
      "position": "top",
      "image": "img/..."
    },

    "safe": {
      "position": "bottom",
      "image": "img/..."
    }
  };

  $scope.tablets.intoxicated.choose = function() {
    if ($scope.gameStates.list.tabletChoosing) {
      if ($scope.currentPlayerIndex == 0) {
        $scope.players[0].score++;
        $scope.players[0].dead = false;
        $scope.players[1].dead = true;
        if (window.cordova) {
          if ($rootScope.config.sound) {
            $rootScope.sound.dead.play();
          }
        }
      }
      else {
        $scope.players[1].score++;
        $scope.players[1].dead = false;
        $scope.players[0].dead = true;
        if (window.cordova) {
          if ($rootScope.config.sound) {
            $rootScope.sound.dead.play();  
          }
        }  
      }
      $scope.gameStates.set('waitingResults');
      $timeout(function() {
        $scope.gameStates.set('gameResults');
      }, $rootScope.config.resultsWaitingTime);
    }
  };

  $scope.tablets.safe.choose = function() {
    if ($scope.gameStates.list.tabletChoosing) {
      if ($scope.currentPlayerIndex == 0) {
        $scope.players[1].score++;
        $scope.players[1].dead = false;
        $scope.players[0].dead = true;
        if (window.cordova) {
          if ($rootScope.config.sound) {
            $rootScope.sound.alive.play();
          }
        }
      }
      else {
        $scope.players[0].score++;
        $scope.players[0].dead = false;
        $scope.players[1].dead = true;
        if (window.cordova) {
          if ($rootScope.config.sound) {
            $rootScope.sound.alive.play();
          }
        }   
      }
      $scope.gameStates.set('waitingResults');
      $timeout(function() {
        $scope.gameStates.set('gameResults');
      }, $rootScope.config.resultsWaitingTime);
    }
  };


  $scope.tablets.changePositions = function() {
    if ($scope.tablets.intoxicated.position == 'top') {
      $scope.tablets.intoxicated.position = 'bottom';
      $scope.tablets.safe.position = 'top';
    }
    else {
      $scope.tablets.intoxicated.position = 'top';
      $scope.tablets.safe.position = 'bottom';    
    }
  };

  $scope.tablets.randomPositions = function() {
    if (Math.floor(Math.random() * 2) == 1) {
      $scope.tablets.changePositions();
    }
  };

  $scope.tablets.randomPositions();




  $scope.acceptPositions = function() {
    // на этом моменте начинается подгрузка рекламы, если нужна реклама на текущем раунде
    if ($scope.currentRound % $rootScope.config.adDivider == 0) {
      currentAdType = $rootScope.getAdType();
      if (currentAdType == 'intersistial') {
        if (window.cordova) {
          admob.cacheInterstitial();
        }
      }
    }
    $scope.gameStates.set('tabletChoosing');
  };


  $scope.showPlayersChangingScreen = function() {
    
    // показываем рекламу, если она нужна на текущем раунде
    if ($scope.currentRound % $rootScope.config.adDivider == 0) {
      if (currentAdType == 'intersistial') {
        if (window.cordova) {
          admob.isInterstitialReady(function(isReady){
            if(isReady){
              admob.showInterstitial();
            }
          });
        }
        $scope.gameStates.set('playersChanging');
      }
      else if (currentAdType == 'self') {
        // показываю свою рекламу
        if (window.cordova) {
          //window.analytics.trackEvent('Adv', 'Show Self Ad');
        }
        $scope.gameStates.set('selfAd');
      }
      else {
        $scope.gameStates.set('playersChanging');
      }
    }
    else {
      $scope.gameStates.set('playersChanging');
    }

  };


  $scope.continueGame = function() {
    if ($scope.currentPlayerIndex == 0) {
      $scope.currentPlayerIndex = 1;
    }
    else {
      $scope.currentPlayerIndex = 0;
    }
    $scope.currentRound++;

    if (window.cordova) {
      //window.analytics.trackView('Game Round ' + $scope.currentRound);
      $rootScope.sound.dead.stop();
      $rootScope.sound.alive.stop();
    }

    $scope.tablets.randomPositions();

    $scope.gameStates.set('tabletsPositions');
  };


  $scope.checkNames = function() {

    // удаление пробелов
    $scope.players[0].name = $scope.players[0].name.replace(/\s+/g, '');
    $scope.players[1].name = $scope.players[1].name.replace(/\s+/g, '');

    // пустые имена
    if ($scope.players[0].name == '') {
      $scope.players[0].name = $scope.players[0].defaultName[$rootScope.config.lang];
    }
    if ($scope.players[1].name == '') {
      $scope.players[1].name = $scope.players[1].defaultName[$rootScope.config.lang];
    }

    // ограничение длины
    if ($scope.players[0].name.length > 10) {
      $scope.players[0].name = $scope.players[0].name.substr(0, 10) + '...';
    }
    if ($scope.players[1].name.length > 10) {
      $scope.players[1].name = $scope.players[1].name.substr(0, 10) + '...';
    }

    // совпадение имен
    if ($scope.players[0].name == $scope.players[1].name) {
      $scope.players[0].name += '1';
      $scope.players[1].name += '2';
    }

    $scope.gameStates.set('firstPlayerScreen');
  };

  $scope.startFirstRound = function() {
    if (window.cordova) {
      //window.analytics.trackView('Game Round ' + $scope.currentRound);
      if ($rootScope.config.sound) {
        $rootScope.sound.round.play();
        $rootScope.sound.bg.setVolume(0.4);
        $timeout(function() {
          $rootScope.sound.bg.setVolume(0.0);
        }, 300);
      }
    }

    $scope.gameStates.set('tabletsPositions');
  };

});


app.controller('RulesController', function($scope, $rootScope) {
  var swiper = new Swiper('.swiper-container', {
      pagination: '.swiper-pagination',
      paginationClickable: true,
      spaceBetween: 0
  });
});


app.controller('TipsController', function($scope, $rootScope) {
  var swiper = new Swiper('.swiper-container', {
    pagination: '.swiper-pagination',
    paginationClickable: true,
    direction: 'vertical'
  });
  if (window.cordova) {
    //window.analytics.trackView('Tips');
  }
});


app.controller('RateController', function($scope, $rootScope) {
  var rated = false;
  
  $scope.rateApp = function(mark) {
    if (!rated) {
      rated = true;
      var stars = document.getElementsByClassName('icon-star');
      for (var i = 0; i < mark; i++) {
        stars[i].style.color = '#7c4dbf';
      }
      if (mark == 5) {
        $scope.rateResultsShow = true;
        // открыть appstore или gp
        if ($rootScope.config.os == 'android') {
          window.open('http://deathpill.lol/android.html', '_blank');
        }
        else if ($rootScope.config.os == 'ios') {
          window.open('http://deathpill.lol/ios.html', '_blank');
        }
      }
      else {
        $scope.rateHateShow = true;
      }
      if (window.cordova) {
        //window.analytics.trackEvent('Events', 'App Rate = ' + mark);
      }
    }
  };

  $scope.hateSend = function() {
    $scope.hateReview = $scope.hateReview.replace(/\s+/g, '');
    if ($scope.hateReview != '') {
      $scope.rateHateShow = false;
      $scope.rateResultsShow = true;
    }
    if (window.cordova) {
      //window.analytics.trackEvent('Events', 'Hate Send');
    }
  };

  if (window.cordova) {
    //window.analytics.trackView('Rate');
  }

});


app.controller('SettingsController', function($scope, $rootScope) {
  $scope.setLanguage = function(lang) {
    $rootScope.config.lang = lang;
    localStorage.setItem('tabletsGameLang', lang);
  };

  $scope.toggleSound = function() {
    if ($rootScope.config.sound) {
      $rootScope.config.sound = false;
      localStorage.setItem('tabletsGameSound', 'false');
      if (window.cordova) {
        $rootScope.sound.bg.setVolume(0.0);

        //window.analytics.trackEvent('Events', 'Sound off');
      }
    }
    else {
      $rootScope.config.sound = true;
      localStorage.setItem('tabletsGameSound', 'true');
      if (window.cordova) {
        $rootScope.sound.bg.setVolume(1.0);

        //window.analytics.trackEvent('Events', 'Sound on');
      }
    }
  };

  if (window.cordova) {
    //window.analytics.trackView('Settings');
  }
});